% A Matlab function implementing the Apriori algorithm
% database... transactional database in the form of boolean matrix, rows
% correspond to transactions, columns correspond to items
function [frequent_itemsets] = my_apriori(database, min_frequency)
% finding frequent itemsets of size 1
candidates = {};
for i = 1:size(database,2)
    candidates{i} = [i];
end
candidates = prune_itemsets(candidates, database, min_frequency);
% variable frequent_itemsets is a cell array containing output of the the
% function, i.e. in then end it should contain all frequent itemsets
frequent_itemsets = candidates;

while ...
    %% You should add the code of the main loop of Apriori here
end

% This is the function Apriori-Gen from the lecture slides. It creates new
% "candidate" itemsets of size k+1 from itemsets of size k. Argument
% old_itemsets is a cell array containing frequent itemsets of size k
function [new_candidates] = apriori_gen(old_itemsets)
candidates_count = length(old_itemsets);
new_candidates = {};
k = 1;
for i = 1:candidates_count
    a = old_itemsets{i};
    for j = (i+1):candidates_count
        b = old_itemsets{j};
        if all(a(1:(end-1)) == b(1:(end-1)))
            new_candidates{k} = [a b(end)];
            k = k+1;
        else
            break;
        end
    end
end

% Here, you should add your code for removing those itemsets I from cell
% array new_candidates such that there exists a subitemset of I which is
% not contined in the cell array old_itemsets (see lecture notes)
% ...

% Function prune_itemsets removoes infrequent itemsets from cell array
% itemsets (it uses an explicit computation of support).
% Input: 
%       itemsets ... a cell array containing itemsets, database is a
%       transactional database (in boolean form), min_frequencey is minimum
%       frequency threshold

function [pruned_itemsets] = prune_itemsets(itemsets, database, min_frequency)
frequencies = compute_frequencies(itemsets, database);
pruned_itemsets = itemsets(frequencies >= min_frequency);

% function for computing relative supports (frequencies) for itemsets
% stored in a cell array itemsets, the frequencies are computed w.r.t.
% database
function [frequencies] = compute_frequencies(itemsets, database)
database_length = size(database,1);
itemsets_count = length(itemsets);
frequencies = ones(1,itemsets_count);
for i = 1:itemsets_count
    itemset = itemsets{i};
    frequencies(i) = sum(all(database(:,itemset)==1,2))/database_length;
end